/**
 * Set of dialogs, beginning with startDialogName.
 */
export interface DialogSet {
  /**
   * The title is displayed at the very top of the dialog screen
   * and usually corresponds to the name of the NPC the player is speaking to.
   */
  title?: string;
  /**
   * Url of image.
   *
   * Should link to image in this repo, to avoid CORS errors.
   */
  imageUrl: string;
  /**
   * At which dialog to start when the dialogSet is first opened by the player.
   */
  startDialogName: string;
  /**
   * All dialogs in this dialogSet.
   */
  dialogs: DialogMap;
}

export type DialogMap = {
  [key: string]: Dialog;
};

/**
 * A dialog situation that a player can encounter in this dialogSet.
 *
 * Principle: NPC says something, user has options how to react.
 */
export interface Dialog {
  /**
   * The title is displayed at the very top of the dialog screen
   * and usually corresponds to the name of the NPC the player is speaking to.
   *
   * Can be used to overwrite title from DialogSet.
   */
  title?: string;
  /**
   * Can be used to overwrite imageUrl from DialogSet.
   * E.g. when a different emotion should be shown for this dialog,
   */
  imageUrl?: string;
  /**
   * What the characters says to the player.
   * Line breaks (\n) are rendered as paragraphs.
   * You may leave this empty or omit it completely.
   */
  text?: string;
  /**
   * Choices of what the user might say in this dialog and where those choices lead to.
   * The player should be given between 0 and 4 options.
   */
  options: DialogOption[];
  /**
   * Upon entering this dialog, register these facts for the user.
   *
   * Facts are texts and may unlock or block dialog options across the game.
   * E.g. The fact "acceptedMissionX" could unlock a dialog option regarding Mission X with another NPC.
   */
  addFacts?: String[];
  /**
   * Upon entering this dialog, remove these facts from the user.
   *
   * Facts are texts and may unlock or block dialog options across the game.
   * E.g. The fact "acceptedMissionX" could unlock a dialog option regarding Mission X with another NPC.
   */
  removeFacts?: String[];
}

export interface DialogOption {
  /**
   * Text on option button.
   * Should be rather short and not have line breaks.
   */
  text: string;
  /**
   * Which dialog in this dialogSet to jump to if this option is chosen.
   */
  linksToDialog: string;
  /**
   * All facts in this array must be registered for this player for this option to show up.
   *
   * Facts are texts and may unlock or block dialog options across the game.
   * E.g. The fact "acceptedMissionX" could unlock a dialog option regarding Mission X with another NPC.
   */
  requiredFacts?: String[];
  /**
   * All facts in this array must *not* be registered for this player for this option to show up.
   *
   * Facts are texts and may unlock or block dialog options across the game.
   * E.g. The fact "acceptedMissionX" could unlock a dialog option regarding Mission X with another NPC.
   */
  forbiddenFacts?: String[];
}

export interface GameState {
  gameFacts: String[];
}

export function isValidGameState(input: unknown): input is GameState {
  if (typeof input !== "object") return false;
  return (
    input["gameFacts"] !== undefined &&
    Array.isArray(input["gameFacts"]) &&
    input["gameFacts"].every((gameFact) => typeof gameFact === "string")
  );
}
